function results = getResults(imResult, imGT)
% GETRESULTS - evaluates the performance on room detection using the
% protocol of Phillips and Chhabbra (PAMI99). This protocol accounts both,
% partial and perfect matches.
%
% INPUT / OUTPUT
%   IMRESULT is the image of the rooms detected for a floor plan. Each room
%   is a connected component of 1's. The background are 0's.
%   IMGT is the corresponding GT image with the GT rooms in the same format
%   as IMRESULT.
%   RESULTS are the results in different rates: the recognition accuracy,
%   the detection rate, the many2one and one2many regarding detections, g_many2one 
%   and one2many regarding GT, the one2one_rate, the number of gt instances,
%   the number of detected instances, the number of exact matches.



% recognition acceptance threshold
acceptance_threshold = 0.5;

% making sure rooms CC are isolated
se1 = strel('Square',5); se2 = strel('Square',3);
imResult = imerode(imResult,se1);
imResult = imdilate(imResult,se2);

% connected components of the images
ccResult = bwconncomp(imResult);
ccGT = bwconncomp(imGT);

% create table
match_score_table = zeros(length(ccResult.PixelIdxList),length(ccGT.PixelIdxList));

%% Step 0. Calculate match_score_table
for i=1:length(ccResult.PixelIdxList),
    imageR = zeros(size(imResult));
    imageR(ccResult.PixelIdxList{i}) = 1;
    for j=1:length(ccGT.PixelIdxList),
        imageG = zeros(size(imGT));
        imageG(ccGT.PixelIdxList{j}) = 1;
        % get the intersection image
        intersectionImage = and(imageR,imageG);
        
        % calculate the matchscore
        matchscore = sum(sum(intersectionImage==1))/max(sum(sum(imageG==1)),sum(sum(imageR==1)));
        
        if matchscore > 0.05,
            match_score_table(i,j) = matchscore;
        end
    end
end

%% Step 1. Calculate matches one2one

one2one_match = zeros(size(match_score_table));
for i=1:size(one2one_match,1),
    for j=1:size(one2one_match,2),
        if (match_score_table(i,j) > acceptance_threshold) &&...
                (match_score_table(i,j) == sum(match_score_table(i,:),2)) ...
                && (match_score_table(i,j) == sum(match_score_table(:,j),1)),
            one2one_match(i,j) = 1;
        end
    end
end 

%% Step 2. Calculate partial one2many matches
match_score_table2 = match_score_table;
match_score_table2(one2one_match==1) = 0;

d_suma = sum(match_score_table2,2);
d_sumaAux = sum(match_score_table2>0,2);
d_suma(d_sumaAux < 2) = 0;

g_suma = sum(match_score_table2,1);
g_sumaAux = sum(match_score_table2>0,1);
g_suma(g_sumaAux < 2) = 0;

d_one2many = sum(d_suma>=acceptance_threshold);
g_one2many = sum(g_suma>=acceptance_threshold);

%% Step 3. Calculate partial many2one matches
d_pairs = [];
g_pairs = [];

for i=1:size(match_score_table2,1),
    for j=1:size(match_score_table2,2),
        d = 0;
        g = 0;
        if (match_score_table2(i,j) > 0) && (d_suma(i) > acceptance_threshold),
            if isempty(d_pairs)
                d_pairs = cat(1,d_pairs,[i,j]);
                d=1;
            elseif sum(d_pairs(:,2)==j,2)==0,
                d_pairs = cat(1,d_pairs,[i,j]);
                d=1;
            end
        end
        if (match_score_table2(i,j) > 0) && (g_suma(j) > acceptance_threshold),
            if isempty(g_pairs),
                g_pairs = cat(1,g_pairs,[i,j]);
                g=1;
            elseif sum(g_pairs(:,1)==i,2)==0,
                g_pairs = cat(1,g_pairs,[i,j]);
                g=1;
            end
        end
        if d==1 && g==1,
            d_pairs(end,:) = [];
            g_pairs(end,:) = [];
        end
    end
end

g_many2one = size(d_pairs,1);
d_many2one = size(g_pairs,1);
one2one = sum(sum(one2one_match));
 
%% Step 8. Calculate final results
rec_accuracy = (one2one+d_one2many+d_many2one)/length(d_suma)*100;
detection_rate = (one2one+g_one2many+g_many2one)/length(g_suma)*100;
one2one_rate = one2one/length(g_suma);

results = {rec_accuracy,detection_rate,d_many2one,d_one2many,...
    g_many2one,g_one2many,one2one_rate,length(g_suma),length(d_suma),one2one};






